@@ -1,8 +1,9 @@
-from typing import Callable, Union, Sequence
+from typing import Callable, Optional, Union, Sequence
 
 import numpy as np
 import torch
 from torch.nn import Module
+from torch.nn.utils import clip_grad_norm_
 from torch.optim import Optimizer
 
 from ..im.utils import identity, dmap, zip_equal, collect
@@ -11,12 +12,20 @@ from .utils import *
 __all__ = 'optimizer_step', 'train_step', 'inference_step', 'multi_inference_step'
 
 
-def optimizer_step(optimizer: Optimizer, loss: torch.Tensor, scaler: torch.cuda.amp.GradScaler = None,
-                   **params) -> torch.Tensor:
+def optimizer_step(
+    optimizer: Optimizer,
+    loss: torch.Tensor,
+    scaler: Optional[torch.cuda.amp.GradScaler] = None,
+    clip_grad: Optional[float] = None,
+    accumulate: bool = False,
+    **params,
+) -> torch.Tensor:
     """
-    Performs the backward pass with respect to ``loss``, as well as a gradient step.
-    If a ``scaler`` is passed - it is used to perform the gradient step (automatic mixed precission support).
+    Performs the backward pass with respect to ``loss``, as well as a gradient step or gradient accumlation.
 
+    If a ``scaler`` is passed - it is used to perform the gradient step (automatic mixed precision support).
+    If a ``clip_grad`` is passed - gradient will be clipped by this value considered as maximum l2 norm.
+    ``accumulate`` indicates whether to perform gradient step or just accumulate gradients.
     ``params`` is used to change the optimizer's parameters.
 
     Examples
@@ -25,30 +34,58 @@ def optimizer_step(optimizer: Optimizer, loss: torch.Tensor, scaler: torch.cuda.
     >>> optimizer_step(optimizer, loss) # perform a gradient step
     >>> optimizer_step(optimizer, loss, lr=1e-3) # set lr to 1e-3 and perform a gradient step
     >>> optimizer_step(optimizer, loss, betas=(0, 0)) # set betas to 0 and perform a gradient step
+    >>> optimizer_step(optimizer, loss, accumulate=True) # perform a gradient accumulation
 
     Notes
     -----
     The incoming ``optimizer``'s parameters are not restored to their original values.
     """
     set_params(optimizer, **params)
-    optimizer.zero_grad()
+
     if scaler is not None:
         # autocast is not recommended during backward
         with torch.cuda.amp.autocast(False):
             scaler.scale(loss).backward()
-            scaler.step(optimizer)
-            scaler.update()
+
+            if not accumulate:
+                if clip_grad is not None:
+                    scaler.unscale_(optimizer)
+                    assert not isinstance(clip_grad, bool), "Use of boolean clip_grad value (e.g. False) can lead to " \
+                                                            "unexpected behaviour. "
+                    clip_grad_norm_(get_parameters(optimizer), clip_grad)
+
+                scaler.step(optimizer)
+                scaler.update()
     else:
         loss.backward()
-        optimizer.step()
+
+        if not accumulate:
+            if clip_grad is not None:
+                clip_grad_norm_(get_parameters(optimizer), clip_grad)
+
+            optimizer.step()
+
+    if not accumulate:
+        optimizer.zero_grad(set_to_none=True)
 
     return loss
 
 
-def train_step(*inputs: np.ndarray, architecture: Module, criterion: Callable, optimizer: Optimizer, n_targets: int = 1,
-               loss_key: str = None, scaler: torch.cuda.amp.GradScaler = None, **optimizer_params) -> np.ndarray:
+def train_step(
+    *inputs: np.ndarray,
+    architecture: Module,
+    criterion: Callable,
+    optimizer: Optimizer,
+    n_targets: int = 1,
+    loss_key: Optional[str] = None,
+    scaler: Optional[torch.cuda.amp.GradScaler] = None,
+    clip_grad: Optional[float] = None,
+    accumulate: bool = False,
+    gradient_accumulation_steps: int = 1,
+    **optimizer_params,
+) -> np.ndarray:
     """
-    Performs a forward-backward pass, and make a gradient step, according to the given ``inputs``.
+    Performs a forward-backward pass, and make a gradient step or accumulation, according to the given ``inputs``.
 
     Parameters
     ----------
@@ -66,10 +103,15 @@ def train_step(*inputs: np.ndarray, architecture: Module, criterion: Callable, o
     loss_key
         in case ``criterion`` returns a dictionary of scalars,
         indicates which key should be used for gradient computation.
-    optimizer_params
-        additional parameters that will override the optimizer's current parameters (e.g. lr).
     scaler
         a gradient scaler used to operate in automatic mixed precision mode.
+    clip_grad
+        maximum l2 norm of the gradient to clip it by.
+    accumulate
+        whether to accumulate gradients or perform optimizer step.
+    gradient_accumulation_steps
+    optimizer_params
+        additional parameters that will override the optimizer's current parameters (e.g. lr).
 
     Notes
     -----
@@ -94,14 +136,31 @@ def train_step(*inputs: np.ndarray, architecture: Module, criterion: Callable, o
         loss = criterion(architecture(*inputs), *targets)
 
     if loss_key is not None:
-        optimizer_step(optimizer, loss[loss_key], scaler=scaler, **optimizer_params)
+        optimizer_step(
+            optimizer,
+            loss[loss_key] / gradient_accumulation_steps,
+            scaler=scaler,
+            clip_grad=clip_grad,
+            accumulate=accumulate,
+            **optimizer_params,
+        )
+
         return dmap(to_np, loss)
 
-    optimizer_step(optimizer, loss, scaler=scaler, **optimizer_params)
+    optimizer_step(
+        optimizer,
+        loss / gradient_accumulation_steps,
+        scaler=scaler,
+        clip_grad=clip_grad,
+        accumulate=accumulate,
+        **optimizer_params,
+    )
+
     return to_np(loss)
 
 
-def inference_step(*inputs: np.ndarray, architecture: Module, activation: Callable = identity) -> np.ndarray:
+def inference_step(*inputs: np.ndarray, architecture: Module, activation: Callable = identity,
+                   amp: bool = False) -> np.ndarray:
     """
     Returns the prediction for the given ``inputs``.
 
@@ -112,12 +171,14 @@ def inference_step(*inputs: np.ndarray, architecture: Module, activation: Callab
     """
     architecture.eval()
     with torch.no_grad():
-        return to_np(activation(architecture(*sequence_to_var(*inputs, device=architecture))))
+        with torch.cuda.amp.autocast(amp or torch.is_autocast_enabled()):
+            return to_np(activation(architecture(*sequence_to_var(*inputs, device=architecture))))
 
 
 @collect
 def multi_inference_step(*inputs: np.ndarray, architecture: Module,
-                         activations: Union[Callable, Sequence[Union[Callable, None]]] = identity) -> np.ndarray:
+                         activations: Union[Callable, Sequence[Union[Callable, None]]] = identity,
+                         amp: bool = False) -> np.ndarray:
     """
     Returns the prediction for the given ``inputs``.
 
@@ -130,14 +191,15 @@ def multi_inference_step(*inputs: np.ndarray, architecture: Module,
     """
     architecture.eval()
     with torch.no_grad():
-        results = architecture(*sequence_to_var(*inputs, device=architecture))
-        if callable(activations):
-            activations = [activations] * len(results)
+        with torch.cuda.amp.autocast(amp or torch.is_autocast_enabled()):
+            results = architecture(*sequence_to_var(*inputs, device=architecture))
+            if callable(activations):
+                activations = [activations] * len(results)
 
-        for activation, result in zip_equal(activations, results):
-            if activation is not None:
-                result = activation(result)
-            yield to_np(result)
+            for activation, result in zip_equal(activations, results):
+                if activation is not None:
+                    result = activation(result)
+                yield to_np(result)
 
 
 @np.deprecate
