@@ -1,10 +1,12 @@
 from pathlib import Path
-from typing import Callable, Union, Iterable
+from typing import Callable, Union, Iterable, Iterator
 
 import numpy as np
 import torch
 from torch import nn
 from torch.optim import Optimizer
+from torch.nn.parameter import Parameter
+from torch.nn.modules.batchnorm import _BatchNorm
 
 from dpipe.io import PathLike
 from dpipe.itertools import squeeze_first, collect
@@ -13,7 +15,7 @@ __all__ = [
     'load_model_state', 'save_model_state',
     'get_device', 'to_device', 'is_on_cuda', 'to_cuda',
     'to_var', 'sequence_to_var', 'to_np', 'sequence_to_np',
-    'set_params', 'set_lr',
+    'set_params', 'set_lr', 'get_parameters', 'has_batchnorm',
     'order_to_mode',
 ]
 
@@ -21,25 +23,26 @@ Device = Union[torch.device, nn.Module, torch.Tensor, str]
 ArrayLike = Union[np.ndarray, Iterable, int, float]
 
 
-def load_model_state(module: nn.Module, path: PathLike, modify_state_fn: Callable = None) -> nn.Module:
+def load_model_state(module: nn.Module, path: PathLike, modify_state_fn: Callable = None, strict: bool = True):
     """
     Updates the ``module``'s state dict by the one located at ``path``.
 
     Parameters
     ----------
-    module
-    path
-    modify_state_fn: Callable(current_state, loaded_state)
+    module: nn.Module
+    path: PathLike
+    modify_state_fn: Callable(current_state, state_to_load)
         if not ``None``, two arguments will be passed to the function:
         current state of the model and the state loaded from the path.
         This function should modify states as needed and return the final state to load.
         For example, it could help you to transfer weights from similar but not completely equal architecture.
+    strict: bool
     """
     state_to_load = torch.load(path, map_location=get_device(module))
     if modify_state_fn is not None:
         current_state = module.state_dict()
         state_to_load = modify_state_fn(current_state, state_to_load)
-    module.load_state_dict(state_to_load)
+    module.load_state_dict(state_to_load, strict=strict)
 
 
 def save_model_state(module: nn.Module, path: PathLike):
@@ -208,3 +211,19 @@ def order_to_mode(order: int, dim: int):
         raise ValueError(f'Invalid order of interpolation passed ({order}) for dim={dim}.')
 
     return mapping[order, dim]
+
+
+def get_parameters(optimizer: Optimizer) -> Iterator[Parameter]:
+    """Returns an iterator over model parameters stored in ``optimizer``."""
+    for group in optimizer.param_groups:
+        for param in group['params']:
+            yield param
+
+
+def has_batchnorm(architecture: nn.Module) -> bool:
+    """Check whether ``architecture`` has BatchNorm module"""
+    for module in architecture.modules():
+        if isinstance(module, _BatchNorm):
+            return True
+
+    return False
